package CoreSim::ALT::ALTRouter;

use strict;

sub new(){
	my $invocant = shift;
	my ($parent, $ipUtility,$altLevel, $_DEBUG) = @_;
	
	my $class = ref($invocant)|| $invocant;
	my $FIB={};
	my $rootIPHash={};
	
	my $self={
		"parent" => $parent,	
		"popNumber" => undef,
		"popIP" => undef,
		"FIB" => $FIB,
		"ipUtility" => $ipUtility,
		"rootIPHash" => $rootIPHash,
		"altLevel" => $altLevel,
		"_DEBUG" => $_DEBUG,
	};
	
	bless($self, $class);
	
	return $self;
}

sub setParent(){
	my $self = shift;
	
	my $parent = shift;
	
	$self->{"parent"} = $parent;
}

sub setChildren(){
	my $self = shift;
	my $children = shift;
	
	$self->{"children"} = $children;
}

sub setPrefix(){
	my $self = shift;
	
	my $prefix = shift;
	$self->{"prefix"} = $prefix;
}

sub setPOPNumber(){
	my $self = shift;
	my $popNumber = shift;
	
	$self->{"popNumber"} = $popNumber;	
}

sub setPOPIP(){
	my $self = shift;
	my $ip = shift;
	
	$self->{"popIP"}= $ip;
}

sub setAS(){
	my $self=shift;
	my ($as) = @_;
	
	$self->{"as"} = $as;
}

sub setALTLevel(){
	my $self = shift;
	my ($altLevel) = @_;
	
	$self->{"altLevel"} = $altLevel;
}

sub setEndorsedByChild(){
	my $self=shift;
	
	$self->{"endorsedByChild"} = 1;	
}

sub addChild(){
	my $self = shift;
	my $child = shift;
	
	my $children = $self->{"children"};
	
	#add the child at the end of the list;
	$children->[scalar(@$children)] = $child;

	#add the entries in FIB
	my $FIB = $self->{"FIB"};
	my $childPrefix = $child->getPrefix();
	$FIB->{$childPrefix} = $child;
	
	
}

sub getPrefix(){
	my $self=shift;
	return $self->{"prefix"};
}

sub getParent(){
	my $self = shift;
	
	return $self->{"parent"};
}

sub getPOPNumber(){
	my $self=shift;
	
	return $self->{"popNumber"};	
}

sub getPOPIP(){
	my $self = shift;
	
	return $self->{"popIP"};
}

sub getChildren(){
	my $self = shift;
	
	return $self->{"children"};
}

sub getAS(){
	my $self=shift;
	
	return $self->{"as"};
}

sub getALTLevel(){
	my $self=shift;
	return $self->{"altLevel"};
}

sub addFIBEntry(){
	my $self=shift;
	
	my ($prefix,$altRouter) = @_;
	
	my $FIB = $self->{"FIB"};
	$FIB->{$prefix} = $altRouter;
}

sub findNextHopForIP(){
	my $self=shift;
	my $searchedIP = shift;

	my $FIB = $self->{"FIB"};
	my $ipUtility = $self->{"ipUtility"};
	my @prefixList = keys  %{$FIB} ;

	my $nextHopAltRouter=undef;
	
	printf("\nIn router %s searching for ip %s\n", $self->{"prefix"}, $searchedIP) if ($self->{"_DEBUG"}>=3);
	#search within the FIB for the prefix which includes the searchedIP
	foreach my $prefix (@prefixList){

		if ($ipUtility->ipInsidePrefix($searchedIP, $prefix)){
				printf("Found more specific prefix %s that includes IP %s in router %s\n", $prefix, $searchedIP, $self->{"prefix"})
					if ($self->{"_DEBUG"}>=3);
				$nextHopAltRouter = $FIB->{$prefix};
				last;
		}
		
	}
	
	#if the searchedIP is not found within FIB 
	if ( !defined($nextHopAltRouter) ) {
		
		#if the current router's prefix includes the searched IP
		if ( $ipUtility->ipInsidePrefix( $searchedIP, $self->{"prefix"} ) ){
			printf("In router %s and the searched IP %s. This is the responsible router\n", $self->{"prefix"}, $searchedIP)
				if ($self->{"_DEBUG"}>=3);
				
			#then we have found the most specific prefix that includes the searchedIP
			return $self;
			
		#if the current router's prefix does not include the searched IP
		}else{
			
			#then we need to ask the "parent" altRouter
			printf("In router %s . Next hop alt router is the parent %s\n", $self->{"prefix"}, $self->{"parent"}->getPrefix())
				if ($self->{"_DEBUG"}>=3);
			$nextHopAltRouter = $self->{"parent"};	
			return $nextHopAltRouter;
		}
	}
	
	
	return $nextHopAltRouter; 
	
	
}

sub isEndorsedByChild(){
	my $self=shift;
	
	if (defined($self->{"endorsedByChild"})){
		return 1;
	}else{
		return 0;
	}
}

sub isRootRouter(){
	my $self=shift;
	
	if (defined( $self->{"rootNode"}) ){
		return 1;
	}else{
		return 0;
	}
}

sub isVirtualRouter(){
	my $self=shift;
	if (!defined($self->{"popIP"})){
		return 1;
	}else{
		return 0;
	}
}

#the next methods are called only for rootRouters
sub addRootRouterInfo(){
	my $self=shift;
	my ($routerInfo) = @_;
	
	my $rootIPHash = $self->{"rootIPHash"};
	
	$self->{"rootNode"} = 1;
	$rootIPHash->{$routerInfo->{"ip"}}->{"popNumber"} = $routerInfo->{"popNumber"};
	$rootIPHash->{$routerInfo->{"ip"}}->{"asNumber"} = $routerInfo->{"asNumber"};
		
}

sub getPOPForIP(){
	my $self=shift;
	my ($ip) = @_;
	
	if (defined ($self->{"rootNode"})){
		return $self->{"rootIPHash"}->{$ip}->{"popNumber"};
	}else{
		return $self->{"popNumber"};
	}
	
}

sub getRootIPList(){
	my $self=shift;
	
	my @ipList = keys %{ $self->{"rootIPHash"} };
	
	return \@ipList;
}

#return 1 
1