#as the name says this is a list. Moreover this is a list of ChordNode objects
package CoreSim::Chord::ChordList;

use strict;
#use CoreSim::Chord::ChordNode;

sub new(){
	
	my $invocant = shift;
	
	my $class = ref($invocant) || $invocant;
	
	my $self = {
		"nodeHash" => {},
	};
	
	bless($self, $class);
	
	return $self;
}

#inserts a node in the list before a given successor
#it keeps the logic of where to insert the node out of this class 
sub insertNode(){
	
	my $self = shift;
	my $newNode = shift;
	my $successor = shift; 
	
	my $nodeHash = $self->{"nodeHash"};
	
	#if it isn't the first node added
	if (defined($successor)){

		my $predecessor = $successor->getPredecessor();

		#add the node to the list
		$newNode->setSuccessor($successor);
		$newNode->setPredecessor($predecessor);
		
		$predecessor->setSuccessor($newNode);
		$successor->setPredecessor($newNode);
		
		#add the node to the nodeHash
		$nodeHash->{$newNode->getChordID()} = $newNode;
		
		my $firstNode = $self->{"firstNode"};
		my $lastNode = $self->{"lastNode"};
	
		#check if the new node may be the first or the last
		if ( $newNode->getChordID() < $firstNode->getChordID() ){
			$self->{"firstNode"} = $newNode;
		}
		if ( $newNode->getChordID() > $lastNode->getChordID() ){
			$self->{"lastNode"} = $newNode;
		}
	
	#it is the first node added 
	}else{
		my $firstNode = $newNode;
		my $lastNode = $newNode;
		
		$firstNode->setSuccessor($lastNode);
		$firstNode->setPredecessor($lastNode);
		
		$lastNode->setSuccessor($firstNode);
		$lastNode->setPredecessor($firstNode);
		
		$nodeHash->{ $newNode->getChordID() } = $newNode;
		$self->{"firstNode"} = $firstNode;
		$self->{"lastNode"} = $lastNode;
	}
	
}

sub deleteNodeID(){
	my $self = shift;
	my $id = shift;
	
	my $nodeHash = $self->{"nodeHash"};

	$nodeHash->{$id}->getPredecessor()->setSuccessor($nodeHash->{$id}->getSuccessor());
	
	delete $nodeHash->{$id};
}

sub deleteNode(){
	my $self = shift;
	my $node = shift;
	
	$node->getPredecessor()->setSuccessor($node->getSuccessor());
	
	my $nodeHash = $self->{"nodeHash"};
	
	delete $nodeHash->{$node->getChordID()};
}

sub getFirstNode(){
	my $self = shift;
	return $self->{"firstNode"};
}

sub getLastNode(){
	my $self = shift;
	return $self->{"lastNode"};
}

sub getNodeHash(){
	my $self = shift;
	return $self->{"nodeHash"};
}

#returns the node with the given chordID
sub getNodeWithID(){
	my $self = shift;
	my $chordID = shift;
	
	return $self->{"nodeHash"}->{$chordID}; 
}

#returns the number of nodes in ChordList
sub getNumberOfNodesInChordList(){
	
	my $self = shift;
	return scalar( keys %{$self->{"nodeHash"}} );
}
#return 1 at the end
1
