package CoreSim::TREE;

use strict;
use CoreSim::TREE::TREEMetrics; 
use CoreSim::TREE::TREETopology;
use CoreSim::TREE::TREEMapResolver; 
use CoreSim::Utilities::IPUtilities; 
 
sub new(){ 
	my $invocant = shift;
	my ($topology, $outputFilePath, $resolverType, $_DEBUG) = @_;
	
	my $class = ref($invocant) || $invocant;
	my $ipUtility = CoreSim::Utilities::IPUtilities->new();
	
	my $self={
		"topology" => $topology,
		"ipUtility" => $ipUtility,
		"resolverType" => $resolverType,
		"outputFilePath" => $outputFilePath,
		"_DEBUG" => $_DEBUG,
	};
	
	bless($self, $class);
	
	&initializeAll($self);
	return $self;
}

sub initializeAll(){
	my $self=shift;
	
	my $topology = $self->{"topology"};
	my $ipUtility = $self->{"ipUtility"};
	my $outputFilePath = $self->{"outputFilePath"};
	my $_DEBUG = $self->{"_DEBUG"};
	my $resolverType = $self->{"resolverType"};
	
	my $treeTopology = CoreSim::TREE::TREETopology->new($topology, $ipUtility, $_DEBUG);
	my $treeMetrics = CoreSim::TREE::TREEMetrics->new($topology, $treeTopology,$ipUtility, $_DEBUG);
	my $treeMapResolver = CoreSim::TREE::TREEMapResolver->new($treeTopology,$topology, $treeMetrics,$ipUtility, $outputFilePath, 
															$resolverType,$_DEBUG);
	
	$self->{"treeTopology"} = $treeTopology;
	$self->{"treeMetrics"} = $treeMetrics;
	$self->{"treeMapResolver"} = $treeMapResolver;
	
}

sub getLookupDelay(){
	my $self=shift;
	my ($itrIP, $searchedIP) = @_;
	
	return $self->{"treeMapResolver"}->searchForIP($itrIP, $searchedIP);
}

sub getResolverType(){
	my $self=shift;
	
	return $self->{"resolverType"};
}

#each mapping system has this function and it uses it to transform a prefix (given as parameter) into an appropriate form
#to be used by the getLookupDelay function. Here from the prefix we determine one of the IPs of the borderPOP and return 
#it as the associatedNode
sub getAssociatedNodeForPrefix(){
	my $self=shift;
	my $prefix = shift;
	
	my $topology = $self->{"topology"};
	my $borderPOP = $topology->getBorderPOPForPrefix($prefix);
	my $associatedNode = $topology->getIPForPOP($borderPOP);
	
	return $associatedNode;
}

sub setNodeIP(){
	my $self=shift;
	my ($altNodeID, $ip) = @_;
	
	my $treeNode = $self->{"treeTopology"}->getLTSWithPrefix($altNodeID);
}

#return 1 at the end
1