package CoreSim::TREE::LTS;

use strict;

sub new(){
	my $invocant = shift;
	my ($ipUtility,$_DEBUG) = @_;
	
	my $class = ref($invocant)|| $invocant;
	my $FIB={};
	my $rootIPHash={};
	my $rerPrefixList = [];
	
	my $self={	
		"popNumber" => undef,
		"popIP" => undef,
		"FIB" => $FIB,
		"ipUtility" => $ipUtility,
		"rootIPHash" => $rootIPHash,
		"rerPrefixList" => $rerPrefixList,
		"_DEBUG" => $_DEBUG,
	};
	
	bless($self, $class);
#	$self->{"_DEBUG"} = 2 ;
	return $self;
}

sub setParent(){
	my $self = shift;
	
	my $parent = shift;
	
	$self->{"parent"} = $parent;
}

sub setRootName(){
	my $self=shift;
	my ($rootName) = @_;
	
	$self->{"rootName"} = $rootName;
}

sub setChildren(){
	my $self = shift;
	my $children = shift;
	
	$self->{"children"} = $children;
}

sub setPrefix(){
	my $self = shift;
	
	my $prefix = shift;
	$self->{"prefix"} = $prefix;
}

sub setPOPNumber(){
	my $self = shift;
	my $popNumber = shift;
	
	$self->{"popNumber"} = $popNumber;	
}

sub setPOPIP(){
	my $self = shift;
	my $ip = shift;
	
	$self->{"popIP"}= $ip;
}

sub setAS(){
	my $self=shift;
	my ($as) = @_;
	
	$self->{"as"} = $as;
}


sub setMS(){
	my $self=shift;
	$self->{"ms"} = 1;
}

sub setLayer(){
	my $self=shift;
	
	my ($layer) = @_;
	$self->{"layer"} = $layer;
}

sub setLocalizationInformation(){
	my $self=shift;
	my ($pop, $as, $ip) = @_;
	
	$self->{"popNumber"} = $pop;
	$self->{"as"} = $as;
	$self->{"popIP"} = $ip;
}

sub addChild(){
	my $self = shift;
	my $child = shift;
	
	#add the entries in FIB
	my $FIB = $self->{"FIB"};
	my $childPrefix = $child->getPrefix();
	$FIB->{$childPrefix} = $child ;
	
}

sub getPrefix(){
	my $self=shift;
	return $self->{"prefix"};
}

sub getParent(){
	my $self = shift;
	
	return $self->{"parent"};
}

sub getPOPNumber(){
	my $self=shift;
	
	return $self->{"popNumber"};	
}

sub getPOPIP(){
	my $self = shift;
	
	return $self->{"popIP"};
}

sub getChildren(){
	my $self = shift;
	
	return $self->{"children"};
}

sub getAS(){
	my $self=shift;
	
	return $self->{"as"};
}

sub getLayer(){
	my $self=shift;
	
	return $self->{"layer"};
}

sub getRootName(){
	my $self=shift;
	
	return $self->{"rootName"};
}

sub getName(){
	my $self=shift;
	return $self->{"name"};	
}


sub findNextHopForIP(){
	my $self=shift;
	my $searchedIP = shift;

	my $FIB = $self->{"FIB"};
	my $ipUtility = $self->{"ipUtility"};
	my @prefixList = keys  %{$FIB} ;
	
	printf("\n(TL) In LTS %s searching for ip %s\n", $self->{"prefix"}, $searchedIP) if ($self->{"_DEBUG"}>=2);
	
	printf("(TL) The size of the fib = %s\n", scalar(keys %$FIB)) if ($self->{"_DEBUG"});
	my $nextHopLTS=undef;
	my $searchedPrefix = "$searchedIP/32";
	
	#the search function is split into 2, depending on the type of router (root or lower layer). Root routers know only about /8 
	#prefixes (RER) and I'm going to use that: an IP address like 1.1.1.1 can be considered the prefix 1.1.1.1/32 and if we decrease the 
	#mask to /8 we will get 1.0.0.0/8 and find directly the searched router (thus no search through the 256 prefix list is needed)
	if (&isRootRouter($self)){
		my $searchedPrefix = $ipUtility->computeParentPrefix($searchedPrefix, 24);
		
		#an L1 LTS should always be found!
		my $l1LTS =  $FIB->{$searchedPrefix} ;
		
		if (defined($l1LTS)){
			printf("(TL)Root %s found the L1 LTS %s responsible for IP %s\n", 
					$self->{"prefix"}, $l1LTS->getPrefix(), $searchedIP) if ($self->{"_DEBUG"} >= 2);
			return $l1LTS;
		}else{
			printf("(TL)(EE) The Root LTS doesn't know all the L1 LTSs!\n");
			
			#this should never happend!(previously I used die here)
			return undef;
		}
	
	#if this is not a root router we start with a prefix /32 and decrease the mask until we find a prefix this router knows about
	#i.e. a prefix which this router has in its FIB. This also assures longest prefix match
	}else{
		
		my $mask = 32;
		my $endLoop = 0;
		
		#mask should be greater than 8
		while ( ($mask>8) && (!$endLoop) ){
			
			my $possibleNextHopLTS = $FIB->{$searchedPrefix};
			
			if(defined($possibleNextHopLTS)){
				$endLoop = 1;
				$nextHopLTS = $possibleNextHopLTS;
				printf("(TL) Found %s as possible next hop LTS for %s\n", $possibleNextHopLTS->getPrefix(), 
						$searchedIP ) if ($self->{"_DEBUG"} >=1);
			}else{
				$searchedPrefix = $ipUtility->computeParentPrefix($searchedPrefix, 1);
				$mask -=1;
			}
		}
		
		if (defined($nextHopLTS)){
			return $nextHopLTS;
		
		#if the current node is responsible for the ip, return the pointer to itself
		}elsif( $ipUtility->ipInsidePrefix( $searchedIP , $self->{"prefix"} ) && (&isMS($self)) ){
			printf("(TL) The LTS %s is responsible for the searched IP %s\n", $self->{"prefix"}, $searchedIP) if ($self->{"_DEBUG"});
			return $self;
		
		#else the node does not know about the ip (this is bad) 
		}else{
			return undef;
		}
		
	}
	
	
}

sub isMS(){
	
	my $self=shift;
	
	if (defined($self->{"ms"})){
		return 1;
	}else{
		return 0;
	}
}
sub isRootRouter(){
	my $self=shift;
	
	if ($self->{"layer"} == 0){
		return 1;
	}else{
		return 0;
	}
}

sub isVirtualRouter(){
	my $self=shift;
	if (!defined($self->{"popIP"})){
		return 1;
	}else{
		return 0;
	}
}

sub setEndorsed(){
	my $self=shift;
	
	$self->{"endorsed"}=1;
}

sub isEndorsed(){
	my $self = shift;
	
	return $self->{"endorsed"};	
}

#the next methods are called only for rootRouters
sub addRootRouterInfo(){
	my $self=shift;
	my ($routerInfo) = @_;
	
	my $rootIPHash = $self->{"rootIPHash"};
	
	$self->{"rootNode"} = 1;
	$rootIPHash->{$routerInfo->{"ip"}}->{"popNumber"} = $routerInfo->{"popNumber"};
	$rootIPHash->{$routerInfo->{"ip"}}->{"asNumber"} = $routerInfo->{"asNumber"};
		
}

sub getPOPForIP(){
	my $self=shift;
	my ($ip) = @_;
	
	if (defined ($self->{"rootNode"})){
		return $self->{"rootIPHash"}->{$ip}->{"popNumber"};
	}else{
		return $self->{"popNumber"};
	}
	
}

sub getASForIP(){
	my $self=shift;
	my ($ip) = @_;
	
	if (defined ($self->{"rootNode"})){
		return $self->{"rootIPHash"}->{$ip}->{"asNumber"};
	}else{
		return $self->{"asNumber"};
	}
}


#used in conjunction with setFIB to copy the FIB from a generic rootRouter to an lts 
sub getFIB(){
	my $self=shift;
	
	return $self->{"FIB"};
}

sub setFIB(){
	my $self=shift;
	my ($FIB) = @_;
	
	$self->{"FIB"}= $FIB;
}


#return 1 
1