package CoreSim::TREE::TREEMapResolverCache;

use strict;

sub new(){
	my $invocant = shift;
	my ($ipUtility, $outputPath,$_DEBUG) = @_;
	
	my $class = ref($invocant) || $invocant;

	my $resolverCache = {};
	my $numberOfEntries =0;
	
	my $self={
		"resolverCache" => $resolverCache,
		"ipUtility" => $ipUtility,
		"outputPath" => $outputPath,
		"numberOfEntries" => $numberOfEntries,
		"_DEBUG" => $_DEBUG,
	};
	
#	$self->{"_DEBUG"} = 3;
	bless($self, $class);
	
	&createOutputFile($self);
	
	return $self;
}

sub findIntermediaryLTS(){
	my $self=shift;
	my ($searchedIP) = @_;
	
	my $ipUtility = $self->{"ipUtility"};
	my $resolverCache = $self->{"resolverCache"};
	
	my $prefix = "$searchedIP/32";
	my $mask=32;
	my $endLoop=0;
	my $intermediaryLTS;
	
	while( ($mask>=8) && (!$endLoop) ){
		
		if (defined($resolverCache->{$prefix})){
			$endLoop=1;
			$intermediaryLTS = $resolverCache->{$prefix};
		}else{
			$mask--;

			$prefix = $ipUtility->computeParentPrefix($prefix,1);
		}
	}
	
	return $intermediaryLTS;
	
}

sub addNodes(){
	my $self=shift;
	my ($peers, $resolverType) = @_;
	
	my $resolverCache  = $self->{"resolverCache"};
	my $length = scalar(@$peers);
	
	#3 types of resolvers: 	0 - iterative [dns iterative+lisp]
	#						1 - recursive [dns recursive + lisp]
	#						2 - modified [dns+lisp]
	if ($resolverType == 0){
		#add the peers to the cache (except for the first node, which might bet the root one or an already cached node)
		for (my $i=1; $i<$length;$i++){
			
			my $peerPrefix = $peers->[$i]->getPrefix();
			$resolverCache->{$peerPrefix} = $peers->[$i];
			$self->{"numberOfEntries"}++;
			
			&writeToOutput($self, $peerPrefix,$self->{"numberOfEntries"} );
			
		}
	}elsif($resolverType == 1){
		#add the L2 LTS (MS) to the cache
		my $peerPrefix = $peers->[scalar(@$peers) - 1]->getPrefix();
		$resolverCache->{$peerPrefix} = $peers->[scalar(@$peers) - 1];
		$self->{"numberOfEntries"}++;
			
		&writeToOutput($self, $peerPrefix,$self->{"numberOfEntries"} );
		
	}elsif( $resolverType == 2 ){
		#add ... nothing to the cache
	
	}elsif( $resolverType == 3){
		#add like in the iterative case 
		
		for (my $i=1; $i<$length;$i++){
			
			my $peerPrefix = $peers->[$i]->getPrefix();
			$resolverCache->{$peerPrefix} = $peers->[$i];
			$self->{"numberOfEntries"}++;
			
			&writeToOutput($self, $peerPrefix,$self->{"numberOfEntries"} );
			
		}
		
	}else{
		printf("(EE)(TMRC) Wrong resolver type. The program will exit!\n");
		die;
	}

	
}

sub createOutputFile(){
	my $self=shift;
	
	my $outputFilePath = $self->{"outputPath"};
	my $outputFile = "$outputFilePath/treeMapCache.txt";
	
	open(OFILE, "> $outputFile") or die printf("Error opening the file %s associated to the ITR!\n", $outputFile);
	printf(OFILE "action\tprefix\tnumberOfEntries\n");
	
	$self->{"cacheFileHandle"} = \*OFILE;
}

sub writeToOutput(){
	my ($self, $prefix, $numberOfEntries) = @_;
	
	my $outputFileHandle = $self->{"cacheFileHandle"};
	printf($outputFileHandle "add\t%s\t%s\n", $prefix, $numberOfEntries);
}


#return 1
1