package CoreSim::TREE::TREEMetrics;

use strict;

sub new(){
	my $invocant = shift;
	my ($topology, $treeTopology,$ipUtility, $_DEBUG) = @_;
	my $class = ref($invocant) || $invocant;
	
	my $numberOfQueriesHash={};
	
	my $self={
		"topology" => $topology, 
		"treeTopology" => $treeTopology,
		"ipUtility" => $ipUtility,
		"numberOfQueriesHash" => $numberOfQueriesHash,
		"_DEBUG" => $_DEBUG,
	};
	
#	$self->{"_DEBUG"}=3;
	bless($self, $class);
	
	return $self;
}

sub getLookupDelayForIP(){
	my $self=shift;
	my($itrIP, $searchedIP) = @_;
	
	printf("(gLDFI) Called with itrIP = %s and searchedIP = %s\n", $itrIP, $searchedIP) if ($self->{"_DEBUG"});
	my $treeTopology = $self->{"treeTopology"};
	
	my ($rootRouter,$rootRouterIP, $latencyToRoot) = &findClosestRootRouter($self, $itrIP);
	printf("The closest Root Router has the ip %s\n", $rootRouterIP) if ($self->{"_DEBUG"});
	
	printf("Finding responsible router for %s starting with the rootRouter with IP %s\n", $searchedIP, $rootRouterIP) 
			if($self->{"_DEBUG"} >=2);
	my ($responsibleLTS, $peers) = &findResponsibleRouter($self, $searchedIP, $rootRouter);
	
}

sub computePathLatency(){
	my ($self,$peers, $mrIP, $resolverType ) = @_;
	
	my $topology = $self->{"topology"};
	
	my $pathLatency = 0;
	my @nodeLatencyList;
	
	#add ITR to the list 
	push(@nodeLatencyList, $mrIP);
	
	#3 types of resolvers: 	0 - iterative [dns iterative+lisp]
	#						1 - recursive [dns recursive + lisp]
	#						2 - modified recursive [dns recursive+lisp]
	#						3 - modified iterative [dns iterative + lisp]
	if ($resolverType == 0){
		#exaple: MR->L1->MR->L2->MR->L3 (this is iterative DNS) + MR->ETR->ITR
		
		my $i;
		
		#compute the sum of latencies between the MR and all the nodes. they are counted twice for RTT
		for ($i=0; $i < scalar (@$peers); $i++ ){
			my $queryLatency = $topology->getLatencyBetweenPOPIPs1($mrIP, $peers->[$i]->getPOPIP() );
			$pathLatency += 2*$queryLatency ;
			
			push(@nodeLatencyList, $queryLatency );
			push(@nodeLatencyList, $peers->[$i]->getPOPIP());
		}
		
		#add the latency (MR, ETR) + (ETR,ITR). given that these are collocated within the simulator MR with ITR and ETR with MS
		my $queryLatency = $topology->getLatencyBetweenPOPIPs1($mrIP, $peers->[$i-1]->getPOPIP() );
		$pathLatency += 2*$queryLatency;
		
		push(@nodeLatencyList, $queryLatency );
		push(@nodeLatencyList, $peers->[$i-1]->getPOPIP());
		
		
	}elsif($resolverType == 1){
		#example: MR->L1->L2->L3->MR (this is recursive DNS) + MR->ETR->ITR
		my $queryLatency  = $topology->getLatencyBetweenPOPIPs1($mrIP, $peers->[0]->getPOPIP());
		$pathLatency += $queryLatency;
		
		push(@nodeLatencyList, $queryLatency );
		push(@nodeLatencyList, $peers->[0]->getPOPIP());
		
		for ( my $i=1; $i< scalar(@$peers) ; $i++){
			$queryLatency = $topology->getLatencyBetweenPOPIPs1($peers->[$i-1]->getPOPIP(), $peers->[$i]->getPOPIP());
			$pathLatency += $queryLatency;
			
			push(@nodeLatencyList, $queryLatency);
			push(@nodeLatencyList, $peers->[$i]->getPOPIP());
		}
		
		
		#latency between the MS and MR (used for the recursive response of the DNS but also as latency betweem MR-ETR and ETR-ITR)
		$queryLatency = $topology->getLatencyBetweenPOPIPs1( $peers->[ scalar(@$peers) -1 ]->getPOPIP(), $mrIP );
		$pathLatency += $queryLatency;
		
		push(@nodeLatencyList, $queryLatency);
		push(@nodeLatencyList, $mrIP);
		
		#the "conversation" between the MR-ETR-ITR
		$pathLatency += 2*$queryLatency;
		
		#MR-ETR
		push(@nodeLatencyList, $queryLatency);
		push(@nodeLatencyList, $peers->[ scalar(@$peers) -1 ]->getPOPIP() );
		
		#ETR-ITR
		push(@nodeLatencyList, $queryLatency);
		push(@nodeLatencyList, $mrIP );
		
	}elsif($resolverType == 2){
		#example: MR->L1->L2->L3->ETR->ITR
		
		my $queryLatency  = $topology->getLatencyBetweenPOPIPs1($mrIP, $peers->[0]->getPOPIP());
		$pathLatency += $queryLatency;
		
		push(@nodeLatencyList, $queryLatency );
		push(@nodeLatencyList, $peers->[0]->getPOPIP());
		
		for ( my $i=1; $i< scalar(@$peers) ; $i++){
			$queryLatency = $topology->getLatencyBetweenPOPIPs1($peers->[$i-1]->getPOPIP(), $peers->[$i]->getPOPIP());
			$pathLatency += $queryLatency;
			
			push(@nodeLatencyList, $queryLatency);
			push(@nodeLatencyList, $peers->[$i]->getPOPIP());
		}
		
		#the latency MS(L3)->ETR is considered 0 and the one ETR-ITR equal with MR-MS
		$queryLatency  = $topology->getLatencyBetweenPOPIPs1($peers->[ scalar(@$peers) - 1 ]->getPOPIP() , $mrIP);
		$pathLatency += $queryLatency;
		push(@nodeLatencyList, $queryLatency);
		push(@nodeLatencyList, $mrIP);
		
	}elsif($resolverType == 3){
		#example: MR->L1->MR->L2->MR->L3(MS)->ETR->ITR
		
		my $i;
		
		#compute the sum of latencies between the MR and all the nodes. they are counted twice for RTT
		for ($i=0; $i < scalar (@$peers); $i++ ){
			my $queryLatency = $topology->getLatencyBetweenPOPIPs1($mrIP, $peers->[$i]->getPOPIP() );
			$pathLatency += 2*$queryLatency ;
			
			push(@nodeLatencyList, $queryLatency );
			push(@nodeLatencyList, $peers->[$i]->getPOPIP());
		}
		
		
	}else{
		printf("(EE) Wrong type of resolver! Program will exit!");
		die;
	}
	
	return ($pathLatency, \@nodeLatencyList); 
}

return 1