package CoreSim::TREE::LTS;

use strict;

sub new(){
	my $invocant = shift;
	my ($ipUtility,$_DEBUG) = @_;
	
	my $class = ref($invocant)|| $invocant;
	my $FIB={};
	my $rootIPHash={};
	my $rerPrefixList = [];
	
	my $self={	
		"popNumber" => undef,
		"popIP" => undef,
		"FIB" => $FIB,
		"ipUtility" => $ipUtility,
		"rootIPHash" => $rootIPHash,
		"rerPrefixList" => $rerPrefixList,
		"_DEBUG" => $_DEBUG,
	};
	
	bless($self, $class);
#	$self->{"_DEBUG"} =3 ;
	return $self;
}

sub setParent(){
	my $self = shift;
	
	my $parent = shift;
	
	$self->{"parent"} = $parent;
}

sub setRootName(){
	my $self=shift;
	my ($rootName) = @_;
	
	$self->{"rootName"} = $rootName;
}

sub setChildren(){
	my $self = shift;
	my $children = shift;
	
	$self->{"children"} = $children;
}

sub setPrefix(){
	my $self = shift;
	
	my $prefix = shift;
	$self->{"prefix"} = $prefix;
}

sub setPOPNumber(){
	my $self = shift;
	my $popNumber = shift;
	
	$self->{"popNumber"} = $popNumber;	
}

sub setPOPIP(){
	my $self = shift;
	my $ip = shift;
	
	$self->{"popIP"}= $ip;
}

sub setAS(){
	my $self=shift;
	my ($as) = @_;
	
	$self->{"as"} = $as;
}

sub setLayer(){
	my $self=shift;
	
	my ($layer) = @_;
	$self->{"layer"} = $layer;
}

sub setRERName(){
	my $self = shift;
	my $name=shift;
	
	$self->{"name"} = $name;
	&setRER($self);
	
	printf("Setting RER = %s\n", $name) if ($self->{"_DEBUG"}>=2);
}

sub setRER(){
	my $self=shift;
	$self->{"RER"} = 1;
}

sub setLocalizationInformation(){
	my $self=shift;
	my ($pop, $as, $ip) = @_;
	
	$self->{"popNumber"} = $pop;
	$self->{"as"} = $as;
	$self->{"popIP"} = $ip;
}

sub addChild(){
	my $self = shift;
	my $child = shift;
	
	my $children = $self->{"children"};
	
	#add the child at the end of the list;
	$children->[scalar(@$children)] = $child;

	#add the entries in FIB
	my $FIB = $self->{"FIB"};
	my $childPrefix = $child->getPrefix();
	push ( @{ $FIB->{$childPrefix} }, $child );
	
	if ($childPrefix eq "RIPE"){
	printf("the child prefix = %s for %s \n", $childPrefix, $self->{"prefix"});
	}
	
	
}

sub addSpecialChild(){
	my $self=shift;
	my ($child, $prefix) = @_;
	
	my $children = $self->{"children"};
	
	#add the child at the end of the list;
	$children->[scalar(@$children)] = $child;

	#add the entries in FIB
	my $FIB = $self->{"FIB"};
	push( @{ $FIB->{$prefix} } , $child );
	
}

sub getPrefix(){
	my $self=shift;
	return $self->{"prefix"};
}

sub getParent(){
	my $self = shift;
	
	return $self->{"parent"};
}

sub getPOPNumber(){
	my $self=shift;
	
	return $self->{"popNumber"};	
}

sub getPOPIP(){
	my $self = shift;
	
	return $self->{"popIP"};
}

sub getChildren(){
	my $self = shift;
	
	return $self->{"children"};
}

sub getAS(){
	my $self=shift;
	
	return $self->{"as"};
}

sub getLayer(){
	my $self=shift;
	
	return $self->{"layer"};
}

sub getRootName(){
	my $self=shift;
	
	return $self->{"rootName"};
}

sub getName(){
	my $self=shift;
	return $self->{"name"};	
}

sub addFIBEntry(){
	my $self=shift;
	
	my ($prefix,$altRouter) = @_;
	
	my $FIB = $self->{"FIB"};
	push( @{ $FIB->{$prefix} }, $altRouter );
}

sub findNextHopForIP(){
	my $self=shift;
	my $searchedIP = shift;

	my $FIB = $self->{"FIB"};
	my $ipUtility = $self->{"ipUtility"};
	my @prefixList = keys  %{$FIB} ;
	
	printf("\n(xR) In router %s searching for ip %s\n", $self->{"prefix"}, $searchedIP) if ($self->{"_DEBUG"}>=2);
	
	printf("(XR) The size of the fib = %s\n", scalar(keys %$FIB)) if ($self->{"_DEBUG"});
	my $nextHopRouterList=undef;
	my $searchedPrefix = "$searchedIP/32";
	
	#the search function is split into 2, depending on the type of router (root or lower layer). Root routers know only about /8 
	#prefixes (RER) and I'm going to use that: an IP address like 1.1.1.1 can be considered the prefix 1.1.1.1/32 and if we decrease the 
	#mask to /8 we will get 1.0.0.0/8 and find directly the searched router (thus no search through the 256 prefix list is needed)
	if (&isRootRouter($self)){
		my $searchedPrefix = $ipUtility->computeParentPrefix($searchedPrefix, 24);
		
		#a RER router should always be found!
		my $rerRouter =  $FIB->{$searchedPrefix}->[0] ;
		
		if (defined($rerRouter)){
			printf("(xR)Root %s found the RER router %s responsible for IP %s\n", 
					$self->getPrefix(), $rerRouter->getName(), $searchedIP) if ($self->{"_DEBUG"} >= 2);
			return $rerRouter;
		}else{
			printf("(xR)(EE) The Root Routers don't know all the RER Routers!\n");
			
			#this should never happend!(previously I used die here)
			return undef;
		}
	
	#if this is not a root router we start with a prefix /32 and decrease the mask until we find a prefix this router knows about
	#i.e. a prefix which this router has in its FIB. This also assures longest prefix match
	}else{
		
		my $mask = 32;
		my $endLoop = 0;
		
		#mask should be greater than 8
		while ( ($mask>9) && (!$endLoop) ){
			
			my $possibleNextHopList = $FIB->{$searchedPrefix};
			
			if(defined($possibleNextHopList)){
				$endLoop = 1;
				$nextHopRouterList = $possibleNextHopList;
				printf("(xR) Found nextHopRouter list associated to prefix %s of length %s\n", $searchedPrefix, 
						scalar(@$nextHopRouterList) ) if ($self->{"_DEBUG"} >=2);
			}else{
				$searchedPrefix = $ipUtility->computeParentPrefix($searchedPrefix, 1);
				$mask -=1;
			}
		}
		
	}
	
	#if for the searchedIP there exists in FIB a wider prefix which includes it then return the router responsible for the prefix
	if ( defined($nextHopRouterList) ) {
		my $listLength = scalar(@$nextHopRouterList);

		my $specificPrefix = "0.0.0.0/0";
		my $nextHopRouter = undef;
		
		#check the list of prefixes and select the most specific one
		for (my $i=0; $i<$listLength; $i++){
			
			my $lts = $nextHopRouterList->[$i];
			my $prefix = $lts->getPrefix();
			
			printf("(xR) Checking prefix %s when searching for ip %s\n", $prefix, $searchedIP) if ($self->{"_DEBUG"}>=2);
			
			my $test = $ipUtility->prefixInsidePrefix($prefix, $specificPrefix);
			
			if($ipUtility->prefixInsidePrefix($prefix, $specificPrefix)){
				$specificPrefix = $prefix;
				$nextHopRouter = $lts;
			}
				
		}
		
		return ($nextHopRouter);
	
	#if no next hop is found	
	}else{
		
		#RERs don't have one prefix (they have more) and the check can't be done as simple as below
		if (&isRERRouter($self)){
			return undef;
		}
		
		my $localPrefix = $self->{"prefix"};
		
		#the current router is responsible for the searchedIP AND it has a layer above 2(for now) which is RER it will return itself
		if( ($ipUtility->ipInsidePrefix($searchedIP, $localPrefix)) && ($self->{"layer"} > 2) ){
#			printf("the router ip %s and the layer %s, while searching for ip %s\n", $self->{"popIP"});
			return $self;
			
		#no next hop is known. return undef
		}else{
			printf("(XR) The %s router can't find the next hop for %s\n", $self->{"prefix"}, $searchedIP);
		
			return undef;
		}
		
	}
	
	
}

sub isRootRouter(){
	my $self=shift;
	
	if (defined( $self->{"rootNode"}) ){
		return 1;
	}else{
		return 0;
	}
}

sub isVirtualRouter(){
	my $self=shift;
	if (!defined($self->{"popIP"})){
		return 1;
	}else{
		return 0;
	}
}

sub isRERRouter(){
	my $self=shift;
	
	return $self->{"RER"};
}

sub setEndorsed(){
	my $self=shift;
	
	$self->{"endorsed"}=1;
}

sub isEndorsed(){
	my $self = shift;
	
	return $self->{"endorsed"};	
}

#the next methods are called only for rootRouters
sub addRootRouterInfo(){
	my $self=shift;
	my ($routerInfo) = @_;
	
	my $rootIPHash = $self->{"rootIPHash"};
	
	$self->{"rootNode"} = 1;
	$rootIPHash->{$routerInfo->{"ip"}}->{"popNumber"} = $routerInfo->{"popNumber"};
	$rootIPHash->{$routerInfo->{"ip"}}->{"asNumber"} = $routerInfo->{"asNumber"};
		
}

sub getPOPForIP(){
	my $self=shift;
	my ($ip) = @_;
	
	if (defined ($self->{"rootNode"})){
		return $self->{"rootIPHash"}->{$ip}->{"popNumber"};
	}else{
		return $self->{"popNumber"};
	}
	
}

sub getASForIP(){
	my $self=shift;
	my ($ip) = @_;
	
	if (defined ($self->{"rootNode"})){
		return $self->{"rootIPHash"}->{$ip}->{"asNumber"};
	}else{
		return $self->{"asNumber"};
	}
}


#used in conjunction with setFIB to copy the FIB from a generic rootRouter to an lts 
sub getFIB(){
	my $self=shift;
	
	return $self->{"FIB"};
}

#this is only a "hack"!
sub addRERPrefix(){
	my $self=shift;
	my $prefix  = shift;
	$self->{"prefix"} = $prefix;
}

sub setFIB(){
	my $self=shift;
	my ($FIB) = @_;
	
	$self->{"FIB"}= $FIB;
}

sub setRootLTS(){
	my $self=shift;
	
	$self->{"rootNode"} = 1;
}

sub getRootIPList(){
	my $self=shift;
	
	my @ipList = keys %{ $self->{"rootIPHash"} };
	
	return \@ipList;
}

#return 1 
1