#This package deals with the "physical" topology. It makes use of other classes to read the data and compute delays, thus it acts like a black box
#for those interestead in just creating a topology and not defining new metrics between AS/POPs or using other input files than those provided
#by iPlane 

package CoreSim::Topology;

use CoreSim::Topology::TopologyMetrics;
use CoreSim::Topology::StoredTopology;
   

use strict;

#constructor 
sub new(){
	my $invocant = shift;
	
	#the file paths come as: ipl, ipm, oa
	my ($xmlRPCServer, $filePaths, $dbiPlane, $dbEstimator,$geoLiteCityFile, $_DEBUG) = @_;
	
	#makes it work with an object or a class name;
	my $class = ref($invocant) || $invocant;
	
	my $self = {
		"xmlRPCServer" => $xmlRPCServer,
		"dbiPlane" => $dbiPlane,
		"dbEstimator" => $dbEstimator, 
		"geoLiteCityFile" => $geoLiteCityFile,
		"_DEBUG" => $_DEBUG,
	};
	
	
	&initializeAll($self, $filePaths);
	
	bless($self, $class);
	return $self;
}

sub initializeAll(){
	
	my $self = shift;
	my $filePaths = shift;
	my $dbiPlane = $self->{"dbiPlane"};
	my $dbEstimator = $self->{"dbEstimator"};
	my $xmlRPCServer = $self->{"xmlRPCServer"};
	my $geoLiteCityFile = $self->{"geoLiteCityFile"};
	my $debug = $self->{"_DEBUG"};
	
	my $storedTopology = CoreSim::Topology::StoredTopology->new($filePaths, $debug);
	
	my $topologyMetrics = CoreSim::Topology::TopologyMetrics->new($storedTopology, $xmlRPCServer, $dbiPlane,$dbEstimator, $geoLiteCityFile, $debug ); 
	
	$self->{"storedTopology"} = $storedTopology;
	$self->{"topologyMetrics"} = $topologyMetrics;
}

sub getPrefixList(){
	
	my $self = shift;
	
	return $self->{"storedTopology"}->getPrefixList2();
}

sub getPOPIPList(){
	
	my $self= shift;
	
	return $self->{"storedTopology"}->getPOPIPList();
}

sub getBorderPOPForAS(){
	
	my $self = shift;
	my $as = shift;

	return $self->{"storedTopology"}->getBorderPOPForAS($as);
}

sub getASForPOP(){
	my $self=shift;
	my $pop = shift;
	
	return $self->{"storedTopology"}->getASForPOP($pop);
}

sub getLatencyBetweenPOPs(){
	
	my $self = shift;
	my $pop1 = shift;
	my $pop2 = shift;
	
	return $self->{"topologyMetrics"}->getLatencyBetweenPOPs($pop1, $pop2);
}

sub getLatencyBetweenPOPIPs(){
	
	my $self = shift;
	my ($ip1,$ip2) = @_; 
	
	return $self->{"topologyMetrics"}->getLatencyBetweenPOPIPs($ip1, $ip2);
}

sub getLatencyBetweenPOPIPs1(){
	my $self=shift;
	my ($ip1,$ip2)=@_;
	
	return $self->{"topologyMetrics"}->getLatencyBetweenPOPIPs1($ip1,$ip2);
}

sub getLatencyBetweenIPs(){
	my $self = shift;
	my ($ip1, $ip2) = @_;
	
	return $self->{"topologyMetrics"}->getLatencyBetweenIPs($ip1,$ip2);
}

sub getXMLRPCQuery(){
	my $self = shift;
	
	my ($ip1, $ip2) = @_;
	
	return $self->{"topologyMetrics"}->xmlRPCQuery($ip1, $ip2);
}

sub getPrefixToAS(){
	
	my $self = shift;
	my $prefix = shift;
	
	return $self->{"storedTopology"}->getPrefixToAS($prefix);
}

sub getIPForPOP(){
	my $self = shift;
	my $pop = shift;
	
	return $self->{"storedTopology"}->getIPForPOP($pop);
}
 
sub getASBorderPOPs(){
	my $self = shift;
	
	return $self->{"storedTopology"}->getAsBorderPOPs();
}

sub getBorderPOPIPList(){
	my $self=shift;
	
	return $self->{"storedTopology"}->getBorderPOPIPList();	
}

sub getASForPrefix(){
	my $self=shift;
	my $prefix = shift;
	
	return $self->{"storedTopology"}->getASForPrefix($prefix);	
}

sub getFilteredPrefixList(){
	my $self=shift;
	
	return $self->{"storedTopology"}->getFilteredPrefixList();
}

sub getBorderPOPForPrefix(){
	my $self=shift;
	my $prefix = shift;
	
	my $as = $self->{"storedTopology"}->getASForPrefix($prefix);
	
	if (defined($as)){
		my $pop = $self->{"storedTopology"}->getBorderPOPForAS($as);
		return $pop;
	}else{
		return undef;
	}
	
}

sub getBestConnectedNPOPs(){
	my $self=shift;
	my ($numberOfPOPs) = @_;
	
	return $self->{"storedTopology"}->getBestConnectedNPOPs($numberOfPOPs);
}

sub getPrefixToRIR(){
	my $self=shift;
	
	return $self->{"storedTopology"}->getPrefixToRIR();
}

sub getRootServers(){
	my $self=shift;
	
	return $self->{"storedTopology"}->getRootServers();
}

#used for testing. if this stays need to change the way it works!
sub readConnectedPOPList(){
	my $self=shift;
	my $inputFilePath = shift;
	
	return $self->{"storedTopology"}->getConnectedPOPList($inputFilePath);
}
#return 1 at the end
1
