package CoreSim::Topology::StoredTopology;

use strict;
use CoreSim::Topology::DataReadModule;

sub new(){
	my $invocant = shift;
	
	#the paths to the files to be read
	my ($filePaths, $_DEBUG) = @_;
	
	my $dataReadModule = CoreSim::Topology::DataReadModule->new($filePaths);
	
	my $class = ref($invocant) || $invocant;
	
	my $self = {
		"dataReadModule" => $dataReadModule,
		"filePaths" => $filePaths,
		"_DEBUG" => $_DEBUG,
	};
	
	bless($self, $class);

	#read all the files
	&readData($self);
	
	return $self;
}

#read all the files
sub readData(){
	my $self = shift;
	
	#array with the paths of the files to be read, if empty the default ones are used (not IMPLEMENTED YET)
	my $paths = shift; 
	
	#list of default file paths
	my $iplFile = "/home/florin/Desktop/pd/iplane/inter_pop_links.txt";
	my $ipmFile = "/home/florin/Desktop/pd/iplane/ip_to_pop_mapping.txt";
	my $oaFile = "/home/florin/Desktop/pd/iplane/origin_as_mapping.txt";
	my $ptrFile = $self->{"filePaths"}->[4];
	my $rsFile = $self->{"filePaths"}->[5];
	
	#read IPL and retrieve the list of structures. If more return values are needed just append the list
	my ($interPOPDelayMap, $asBorderPOPs,$popToASMapping, $interASDelayMap) = $self->{"dataReadModule"}->readIPLData($iplFile);
	my ($ipPOPMapping, $popIPMapping) = $self->{"dataReadModule"}->readIPMData($ipmFile);
	my ($prefixToAS, $asToPrefix) = $self->{"dataReadModule"}->readOAData($oaFile);
	my ($filteredPrefixList) = $self->{"dataReadModule"}->readFilteredPrefixList();
	my ($prefixToRIR) = $self->{"dataReadModule"}->readPrefixToRIR($ptrFile);
	my ($rootServers) = $self->{"dataReadModule"}->readRootServers($rsFile);
	
	#append the structures into the object hash
	$self->{"interPOPDelayMap"} = $interPOPDelayMap;
	$self->{"ipPOPMapping"} = $ipPOPMapping;	
	$self->{"popIPMapping"} = $popIPMapping;
	$self->{"asBorderPOPs"} = $asBorderPOPs;
	$self->{"prefixToAS"} = $prefixToAS;
	$self->{"asToPrefix"} = $asToPrefix;
	$self->{"popToASMapping"} = $popToASMapping;
	$self->{"interASDelayMap"} = $interASDelayMap;
	$self->{"filteredPrefixList"} = $filteredPrefixList;
	$self->{"prefixToRIR"} = $prefixToRIR;
	$self->{"rootServers"} = $rootServers;
	
#	printf("Undef the data read module!\n");
#	undef($self->{"dataReadModule"});
}

#getter for the interPOPDelayMap
sub getInterPOPDelayMap(){
	my $self = shift;
	return $self->{"interPOPDelayMap"};
}

sub getInterASDelayMap(){
	my $self=shift;
	return $self->{"interASDelayMap"};	
}

#getter for the asBorderPOPs hash
sub getAsBorderPOPs(){
	my $self = shift;
	return $self->{"asBorderPOPs"};
}

#getter for the prefix list
sub getPrefixList(){
	my $self = shift;
	
	#return the keys from the prefixToAS hash
	my @list = keys %{ $self->{"prefixToAS"} };
	
	return \@list;
}

#this constructs the prefix list from the inter_pop_links.txt
sub getPrefixList2(){
	my $self = shift;
	
	my @asList =  keys %{$self->{"asBorderPOPs"}};
	my $asToPrefix = $self->{"asToPrefix"};
	my @list;
	
	my $notDefined=0;
	my $nbPrefix =0;
	my $nbAS = 0;
	
	#extract from the prefix list read from oa file just those AS-es that are found in the inter_pop_links.txt file 
	foreach my $as (@asList){
		if ( defined($asToPrefix->{$as}) ) {
			
			#take all the prefixes for an AS
			my @prefixKeys = keys %{$asToPrefix->{$as}};
			
			#and add them to the list
			foreach my $prefixKey (@prefixKeys){
				push(@list, $prefixKey );
				$nbPrefix++;
			} 
			$nbAS++;
		
		#this prefix was not defined in asToPrefix
		}else{
			$notDefined++;
		}
	}
	
	my $totalNbOfPrefixes = scalar( keys %{ $self->{"prefixToAS"} } );
	my $totalNbOfASs = scalar( keys %{ $self->{"asToPrefix"} } );
	
	printf("There are %d as-es that appear in inter_pop_links and do not appear in origin_as_mapping\n", $notDefined);
	printf("We use %s prefixes from %s ASes to build the prefix list\n", $nbPrefix, $nbAS);
	printf("The total number of prefixes is %s, thus we don't use %s prefixes\n", $totalNbOfPrefixes, $totalNbOfPrefixes-$nbPrefix);
	printf("The total number of ASs %s and we use %s. The number of unused ASs %s\n",$totalNbOfASs, $nbAS, $totalNbOfASs - $nbAS );
	
	return \@list;
}

#returns the filteredPrefixList(used by both the Chord and ALT)
sub getFilteredPrefixList(){
	my $self=shift;
	
	if (defined($self->{"filePaths"}->[3]) ){
		return $self->{"filteredPrefixList"};
	}else{
		printf("it was undef!\n");
		return undef; 
	}
}

#getter for the ip to PoP mapping hash
sub getIPPOPMapping(){
	my $self = shift;
	return $self->{"ipPOPMapping"};
}

sub getPOPIPMapping(){
	my $self = shift;
	return $self->{"popIPMapping"};
}

#returns the first IP for a given POP ... used for testing
sub getIPForPOP(){
	my $self = shift;
	my $popNumber = shift;

	return $self->{"popIPMapping"}->{$popNumber}->[0];
}

sub getIPListForPOP(){
	my $self=shift;
	my $pop = shift;
	
	my $ipList = $self->{"popIPMapping"}->{$pop};
		
	return $ipList;
}

sub getPOPForIP(){
	my $self = shift;
	my $ip = shift;
	
	return $self->{"ipPOPMapping"}->{$ip};
}

sub getPrefixToAS(){
	my $self = shift;
	my $prefix = shift;
	
	return $self->{"prefixToAS"}->{$prefix};
}

sub getASForPOP(){
	my $self = shift;
	my $pop = shift;
	
	return $self->{"popToASMapping"}->{$pop};	
}

#returns the border POP for a given AS. 
#In case there are more than one border POPs per AS the one with the most inter-as connections is returned
sub getBorderPOPForAS(){
	my $self = shift;
	my $as = shift;
	
	#get the hash with all the POPs for a given AS
	my $list = $self->{"asBorderPOPs"}->{$as};
	
	if (!defined($list)){
		return undef;
	}
	my $max = 0;
	my $bestPOP;
	
	#find the POP with the max number of inter-as connections 
	foreach my $pop (keys %$list){
		
		if ($list->{$pop} > $max) {
			$max = $list->{$pop};
			$bestPOP = $pop;
		}
	}
	
	
	return $bestPOP;
	
}

#only for testing! it uses a connected graph of pops(obtained with the Graph module)
sub getConnectedPOPList(){
	my $self=shift;
	my $inputFilePath = shift;
	
	return $self->{"dataReadModule"}->readPOPConnectedList($inputFilePath);
}

sub getPOPIPList(){
	my $self = shift;
	
	my $list = [ keys %{ $self->{"ipPOPMapping"} } ];
	return $list;
}

#returns the border POP IP list with constrains related to the Transit AS-es. MIGHT BE BUGGY!
sub getBorderPOPIPList(){
		
		my $self=shift;
		
		#take the AS list from asBorderPOPs
		my $asList = [ keys %{$self->{"asBorderPOPs"}} ];
		my $borderPOPIPList;
		
		foreach my $as (@$asList){
				push(@$borderPOPIPList, &getBorderPOPForAS($self, $as));
		}

		return $borderPOPIPList;
		
} 

sub getASForPrefix(){
	my $self=shift;
	
	my $prefix = shift;
	
	return $self->{"prefixToAS"}->{$prefix};
}

#returns a list of hashes with pop, as and ip information ordered by the number of connections 
sub getBestConnectedNPOPs(){
	my $self=shift;
	my ($numberOfPOPs) = @_;
	
	my @allASBorderPOPs;
	my %borderPOPsHash;
	
	#border pops exist only for the ASs that apear in asBorderPOPs
	my @asList = keys %{$self->{"asBorderPOPs"}};
	

	foreach my $as (@asList){
		my $borderPOP = &getBorderPOPForAS($self, $as);
		my $nbOfLinks = $self->{"asBorderPOPs"}->{$as}->{$borderPOP};
		
		push(@allASBorderPOPs, $borderPOP);
	
		$borderPOPsHash{$borderPOP} = $nbOfLinks;
		
		
	} 
	
	
	my @sortedASBorderPOPs = sort{ $borderPOPsHash{$b} <=> $borderPOPsHash{$a} } @allASBorderPOPs;
	
#	printf("Test border pop list\n @sortedASBorderPOPs\n");
#	printf( "TEST values $borderPOPsHash{3342},$borderPOPsHash{390},$borderPOPsHash{792716},$borderPOPsHash{81848}\n "); 	

	my @returnList;
	for (my $i=0;$i<$numberOfPOPs; $i++){
		
		my %hashElement;
		$hashElement{"ip"} = &getIPForPOP($self, $sortedASBorderPOPs[$i]);
		$hashElement{"popNumber"} = $sortedASBorderPOPs[$i];
		$hashElement{"asNumber"} = &getASForPOP($self, $sortedASBorderPOPs[$i]);
		
		push(@returnList, \%hashElement);
	}
	
	return \@returnList;
}

sub getPrefixToRIR(){
	my $self=shift;
	
	return $self->{"prefixToRIR"};
}

sub getRootServers(){
	my $self=shift;
	
	return $self->{"rootServers"};
}

#return 1
1
