package CoreSim::Utilities::Estimator;

use Geo::IP;  
use strict;

my $pi = atan2(1,1) * 4;

sub new(){
	
	my $invocant = shift;
	my ($geoLiteCityFile) = @_;
	
	my $class = ref($invocant) || $invocant;
	
	my $gi = Geo::IP->open($geoLiteCityFile,GEOIP_STANDARD);
	
	my $self={
		"gi" => $gi,
	};
	
	bless($self, $class);
	
	return $self;
}

sub getLatency {
	my $self=shift;
	my ($ip1,$ip2) = @_;

	my $gi = $self->{"gi"};
	
	my $g1 = $gi->record_by_name($ip1);
	my $g2 = $gi->record_by_name($ip2);

	my $dist;
	
	if (defined($g1) && defined($g2)) {
		if (($g1->latitude == $g2->latitude) && ($g1->longitude==$g2->longitude)) {return 0;}
		$dist = &distance($g1->latitude,$g1->longitude,$g2->latitude,$g2->longitude,"K");
	
	}else{
		$dist = rand()*20000;
	}

	return int( ($dist*0.017203)+65.968 ); # Results are in ms
}


sub distance {
	
	my ($lat1, $lon1, $lat2, $lon2, $unit) = @_;
	my $theta = $lon1 - $lon2;
	my $dist = sin(deg2rad($lat1)) * sin(deg2rad($lat2)) + cos(deg2rad($lat1)) * cos(deg2rad($lat2)) * cos(deg2rad($theta));
	
	$dist = acos($dist);
	$dist = rad2deg($dist);
	$dist = $dist * 60 * 1.1515;
	
	if ($unit eq "K") {
		$dist = $dist * 1.609344;
	}elsif ($unit eq "N") {
		$dist = $dist * 0.8684;
	}
	return ($dist);
}

sub acos {
	my ($rad) = @_;
	my $ret = atan2(sqrt(1 - $rad**2), $rad);
	return $ret;
}

sub deg2rad {
	my ($deg) = @_;
	
	return ($deg * $pi / 180);
}

sub rad2deg {
	my ($rad) = @_;
	return ($rad * 180 / $pi);
}

#return 1 at the end
1
