package CoreSim::Utilities::MCLinkedList;

use strict;

sub new(){
	
	my $invocant = shift;
	my ($ipUtility, $_DEBUG) = @_;
	
	my $class = ref($invocant) || $invocant;
	
	my $first = {"next" =>undef ,"prev"=>undef, "timestamp" => undef, "prefix" => "first"};
	my $last = {"next" =>undef ,"prev"=>undef, "timestamp" => undef, "prefix" => "last"};
	$first->{"next"} = $last;
	$last->{"prev"} = $first;
	
	my $prefixDatabase = undef;
	
	
	my $self={
		"first" => $first,
		"last" => $last,
		"ipUtility" => $ipUtility,
		"prefixDatabase" => $prefixDatabase,
		"_DEBUG" => $_DEBUG,
	
	};
	
#	$self->{"_DEBUG"}=3;
	bless($self, $class);
	
	return $self;
}

sub pushElement(){
	my $self = shift;
	my ($timestamp, $prefix) = @_;
	
	my $first = $self->{"first"};
	my $last = $self->{"last"};
	 
	printf("The list before pushing the element %s:\n", $prefix) if ($self->{"_DEBUG"}>3);
	&printTheList($self) if ($self->{"_DEBUG"}>3);
	
	printf("(MCLL) inserting %s with timestamp %s in the linked list\n", $prefix, $timestamp->getTimestampString()) if ($self->{"_DEBUG"}>=2);
	
	#new last element. connect it to the last one
	my $newLast = {"next" =>undef ,"prev"=>undef, "timestamp" => undef, "prefix" => "last" };
	$newLast->{"prev"} = $self->{"last"};
	
	#the old last becomes the new element
	$self->{"last"}->{"timestamp"} = $timestamp;
	$self->{"last"}->{"prefix"} = $prefix;
	$self->{"prefixDatabase"}->{$prefix} = $self->{"last"};
	
	#connect the old last to new last
	$self->{"last"}->{"next"} = $newLast;
	$self->{"last"} = $newLast;
	
	printf("The list after pushing the element %s\n", $prefix) if ($self->{"_DEBUG"}>3);
	&printTheList($self) if ($self->{"_DEBUG"}>3);
	
}

sub firstElement(){
	my $self=shift;
	
	my $first=$self->{"first"}->{"next"};
	
	if ($first->{"prefix"} eq $self->{"last"}->{"prefix"}){
		return (undef, undef);
	}else{
		#return the first object
		return ( $first->{"prefix"}, $first->{"timestamp"});
	}
} 

sub deleteFirstElement(){
	my $self=shift;
	
	printf("(MCLL) Deleting the first element from the list with prefix %s and timestamp %s\n", $self->{"first"}->{"next"}->{"prefix"}, 
			$self->{"first"}->{"next"}->{"timestamp"}->getTimestampString()) if ($self->{"_DEBUG"}>2);
	
	printf("the list before deleting the first element\n")if ($self->{"_DEBUG"}>3);		
	&printTheList($self) if ($self->{"_DEBUG"}>3);
	
	my $first = $self->{"first"}->{"next"};	
	$first->{"next"}->{"prev"} = $self->{"first"};
	$self->{"first"}->{"next"}=$first->{"next"};
	
	
	#delete the first element
	delete($self->{"prefixDatabase"}->{ $first->{"prefix"} } );
	undef(%$first);
	
	if ($self->{"_DEBUG"}>3){
		printf("(MCLL) The prefix database:");
		foreach my $prefix (keys %{$self->{"prefixDatabase"}}){
			printf("%s ", $prefix);
		}	
		printf("\n");
	}
	
	
	printf("the list after deleting the first element\n")if ($self->{"_DEBUG"}>3);
	&printTheList($self) if ($self->{"_DEBUG"}>3) ;
}

sub printTheList(){
	my $self=shift;
	
	my $first = $self->{"first"};
	my $last = $self->{"last"};
	my $element = $first->{"next"};

	while($element->{"prefix"} ne $self->{"last"}->{"prefix"}){
			printf("%s ", $element->{"prefix"});
			$element = $element->{"next"};
	};
	
	
	printf("\n");
}

sub updateElement(){
	my $self=shift;
	my ($prefix, $newTimestamp) = @_;
	
	my $element = $self->{"prefixDatabase"}->{$prefix};
	
	printf("(MCLL) updating %s with timestamp %s to the timestamp %s\n", $prefix, $element->{"timestamp"}->getTimestampString(), 
			$newTimestamp->getTimestampString()) if ($self->{"_DEBUG"}>=2);
	
	
	printf("(MCLL) The list before updating the element %s\n", $prefix) if ($self->{"_DEBUG"}>3);
	&printTheList($self) if ($self->{"_DEBUG"}>3);

	#delete the element from the list
	$element->{"prev"}->{"next"} = $element->{"next"};
	$element->{"next"}->{"prev"} = $element->{"prev"};
	
	#create new last and connect it to the old last
	my $newLast = {"next" =>undef ,"prev"=>undef, "timestamp" => undef, "prefix" => "last" };
	$newLast->{"prev"} = $self->{"last"};
	
	#the old last becomes the new element
	$self->{"last"}->{"timestamp"} = $newTimestamp;
	$self->{"last"}->{"prefix"} = $prefix;
	
	$self->{"prefixDatabase"}->{$prefix} = $self->{"last"};
	
	#connect the old last to new last
	$self->{"last"}->{"next"} = $newLast;
	$self->{"last"} = $newLast;
	
	#delete the element
	undef(%$element);
	
	printf("(MCLL) The list after updating the element %s\n", $prefix) if ($self->{"_DEBUG"}>3);
	&printTheList($self)if ($self->{"_DEBUG"}>3);

}

sub elementExists(){
	my ($self, $prefix) = @_;
	
	if( $self->{"_DEBUG"} > 3) {
		printf("(MCLL-EE) prefix database, searching for %s :\n", $prefix);
		foreach my $key (keys %{$self->{"prefixDatabase"} }){
			printf("%s ", $key);
		}
		printf("\n");
	}
	
	return defined( $self->{"prefixDatabase"}->{$prefix} );

}

sub getTimestamp(){
	my ($self, $prefix) = @_;
		
	if ( defined($self->{"prefixDatabase"}->{$prefix}) ){
		return $self->{"prefixDatabase"}->{$prefix}->{"timestamp"};
	}else{
		return undef;
	}
	
}

#return 1 at the end
return 1